"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
var validator_error_1 = require("./errors/validator-error");
var validation_1 = require("./validation");
var validation_context_1 = require("./validation-context");
var validation_message_1 = require("./validation-message");
var validation_message_collection_1 = require("./validation-message-collection");
var defensive_programming_framework_1 = require("defensive-programming-framework");
/**
 * The validatable extensions.
 *
 * @export
 * @abstract
 * @class ValidatableExtensions
 */
var ValidatableExtensions = /** @class */ (function () {
    function ValidatableExtensions() {
    }
    // #region Public Static Methods (2)
    /**
    * Validates the specified property for the specified validation source. If no property name is provided, all properties are validated.
    *
    * @param {string} [propertyName] - The validation source.
    * @param {string} [propertyName] - The property name.
    * @param {string[]} [validationContexts] - The validation contexts.
    * @returns {ValidationMessageCollection} - The validation messages.
    */
    ValidatableExtensions.validate = function (validationSource, propertyName, validationContexts) {
        defensive_programming_framework_1.cannotBeNull(validationSource);
        propertyName = defensive_programming_framework_1.whenIsNullOrWhiteSpace(propertyName, null);
        validationContexts = defensive_programming_framework_1.whenIsNull(validationContexts, []);
        if (defensive_programming_framework_1.isNull(propertyName)) {
            return ValidatableExtensions.validateObject(validationSource, validationContexts);
        }
        else {
            return ValidatableExtensions.validateProperty(validationSource, propertyName, validationContexts);
        }
    };
    // #endregion
    // #region Private Static Methods (4)
    /**
     * Get the list of validators for the specified property in the specified validation source.
     *
     * @private
     * @static
     * @param {IValidatable} validationSource- The validation source.
     * @param {string} propertyName - The property name.
     * @returns {Validator[]} - The list of validators.
     */
    ValidatableExtensions.getValidators = function (validationSource, propertyName) {
        defensive_programming_framework_1.cannotBeNull(validationSource);
        defensive_programming_framework_1.cannotBeNullOrEmpty(propertyName);
        return Reflect.getMetadataKeys(validationSource, propertyName)
            .filter(function (x) { return typeof (x) === "string" && validation_1.Validation.isValidValidatorKey(x); })
            .map(function (x) { return Reflect.getMetadata(x, validationSource, propertyName); });
    };
    /**
     * Validates the validators for the specified property, property value and validation context in the specified validation source.
     *
     * @private
     * @static
     * @param {IValidatable} validationSource - The validation source.
     * @param {string} propertyName - The property name.
     * @param {*} propertyValue - The property value.
     * @param {(string | null)} validationContext - The validation context.
     * @returns {ValidationMessageCollection} - The validation messages.
     */
    ValidatableExtensions.validateValidators = function (validationSource, propertyName, propertyValue, validationContext) {
        defensive_programming_framework_1.cannotBeNull(validationSource);
        defensive_programming_framework_1.cannotBeNullOrEmpty(propertyName);
        var validationMessages = [];
        var isValid = false;
        for (var _i = 0, _a = ValidatableExtensions.getValidators(validationSource, propertyName); _i < _a.length; _i++) {
            var validator = _a[_i];
            if (validator.validationContext === validationContext) {
                try {
                    isValid = validator.isValid(propertyValue);
                }
                catch (ex) {
                    throw new validator_error_1.ValidatorError("Unhandled validator error occurred.", typeof validator, typeof validationSource, propertyName, ex);
                }
                if (!isValid) {
                    validationMessages.push(new validation_message_1.ValidationMessage(validator.message, validationSource, propertyName, validator.validationLevel, validator.validationContext, validator.validationPriority));
                }
            }
        }
        return new validation_message_collection_1.ValidationMessageCollection(validationMessages);
    };
    /**
     * Validates the specified validation source for the specified validation contexts.
     *
     * @private
     * @static
     * @param {IValidatable} validationSource - The validation source.
     * @param {string[]} validationContexts- The validation contexts.
     * @returns - The validation messages.
     */
    ValidatableExtensions.validateObject = function (validationSource, validationContexts) {
        defensive_programming_framework_1.cannotBeNull(validationSource);
        var validationMessages = [];
        var propertyNames = Object.keys(validationSource);
        for (var _i = 0, propertyNames_1 = propertyNames; _i < propertyNames_1.length; _i++) {
            var propertyName = propertyNames_1[_i];
            for (var _a = 0, _b = ValidatableExtensions.validateProperty(validationSource, propertyName, validationContexts).toArray(); _a < _b.length; _a++) {
                var message = _b[_a];
                validationMessages.push(message);
            }
        }
        return new validation_message_collection_1.ValidationMessageCollection(validationMessages);
    };
    /**
     * Validates the specified property in the specified validation source for the specified validation contexts.
     *
     * @private
     * @static
     * @param {IValidatable} validationSource - The validation source.
     * @param {string} propertyName - The property name.
     * @param {((string | null)[])} validationContexts - The validation contexts.
     * @returns {ValidationMessageCollection} - The validation messages.
     */
    ValidatableExtensions.validateProperty = function (validationSource, propertyName, validationContexts) {
        defensive_programming_framework_1.cannotBeNull(validationSource);
        defensive_programming_framework_1.cannotBeNullOrEmpty(propertyName);
        var validationMessages = [];
        var propertyValue;
        var propertyNames = Object.getOwnPropertyNames(validationSource);
        if (propertyNames.some(function (x) { return x === propertyName; })) {
            propertyValue = Reflect.get(validationSource, propertyName);
            validationContexts = validationContexts;
            validationContexts.push(validation_context_1.ValidationContext.default); // add default context
            validationContexts = validationContexts.filter(function (value, index, self) { return self.indexOf(value) === index; }); // distinct
            for (var _i = 0, validationContexts_1 = validationContexts; _i < validationContexts_1.length; _i++) {
                var validationContext = validationContexts_1[_i];
                for (var _a = 0, _b = ValidatableExtensions.validateValidators(validationSource, propertyName, propertyValue, validationContext).toArray(); _a < _b.length; _a++) {
                    var message = _b[_a];
                    validationMessages.push(message);
                }
            }
        }
        return new validation_message_collection_1.ValidationMessageCollection(validationMessages);
    };
    return ValidatableExtensions;
}());
exports.ValidatableExtensions = ValidatableExtensions;
//# sourceMappingURL=validatable-extensions.js.map