"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var validation_level_1 = require("./validation-level");
var defensive_programming_framework_1 = require("defensive-programming-framework");
/**
 * The validation message collection.
 *
 * @export
 * @class ValidationMessageCollection
 */
var ValidationMessageCollection = /** @class */ (function () {
    // #endregion
    // #region Constructors (1)
    /**
     * Creates an instance of ValidationMessageCollection.
     * @param {ValidationMessage[]} validationMessages
     */
    function ValidationMessageCollection(validationMessages) {
        var _this = this;
        // #region Properties (1)
        /**
         * The validation messages.
         *
         * @private
         * @type {ValidationMessage[]}
         */
        this.validationMessages = [];
        validationMessages = validationMessages ? validationMessages : [];
        // make a copy of the array
        validationMessages = validationMessages.concat([]);
        // remove null and undefined values
        for (var i = validationMessages.length - 1; i >= 0; i--) {
            if (defensive_programming_framework_1.isNull(validationMessages[i])) {
                validationMessages.splice(i, 1);
            }
        }
        // remove duplicates
        for (var i = validationMessages.length - 1; i >= 0; i--) {
            for (var j = i - 1; j >= 0; j--) {
                if (validationMessages[i].validationSource === validationMessages[j].validationSource &&
                    validationMessages[i].propertyName === validationMessages[j].propertyName &&
                    validationMessages[i].validationLevel === validationMessages[j].validationLevel &&
                    validationMessages[i].validationPriority === validationMessages[j].validationPriority &&
                    validationMessages[i].validationContext === validationMessages[j].validationContext &&
                    validationMessages[i].message === validationMessages[j].message) {
                    validationMessages.splice(i, 1);
                    break;
                }
            }
        }
        // order
        validationMessages = validationMessages.sort(function (a, b) {
            var diff;
            diff = -_this.compare(Math.max(a.validationPriority, 0), Math.max(b.validationPriority)); // order by validation priority (lower priority means higher score, priority cannot be lower than 0)
            if (diff === 0) {
                diff = _this.compare(_this.validationLevelToNumber(a.validationLevel), _this.validationLevelToNumber(b.validationLevel)); // then by validation level
            }
            diff = -diff; // descending
            return diff;
        });
        this.validationMessages = validationMessages;
    }
    Object.defineProperty(ValidationMessageCollection.prototype, "errors", {
        // #endregion
        // #region Public Accessors (8)
        /**
         * Gets the list of error validation messages.
         *
         * @readonly
         */
        get: function () {
            return new ValidationMessageCollection(this.validationMessages.filter(function (validationMessage) { return validationMessage.validationLevel === validation_level_1.ValidationLevel.error; }));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidationMessageCollection.prototype, "first", {
        /**
         * Gets the first validation message in the collection.
         *
         * @readonly
         */
        get: function () {
            return this.validationMessages.length > 0 ? this.validationMessages[0] : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidationMessageCollection.prototype, "hasErrors", {
        /**
         * Checks if the collection contains any error validation messages.
         *
         * @returns {boolean} - True if the validation message collection contains error messages; false otherwise.
         */
        get: function () {
            return this.validationMessages.some(function (validationMessage) { return validationMessage.validationLevel === validation_level_1.ValidationLevel.error; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidationMessageCollection.prototype, "hasInfos", {
        /**
         * Checks if the collection contains any info validation messages.
         *
         * @returns {boolean} - True if the validation message collection contains info messages; false otherwise.
         */
        get: function () {
            return this.validationMessages.some(function (validationMessage) { return validationMessage.validationLevel === validation_level_1.ValidationLevel.info; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidationMessageCollection.prototype, "hasWarnings", {
        /**
         * Checks if the collection contains any warning validation messages.
         *
         * @returns {boolean} - True if the validation message collection contains warning messages; false otherwise.
         */
        get: function () {
            return this.validationMessages.some(function (validationMessage) { return validationMessage.validationLevel === validation_level_1.ValidationLevel.warning; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidationMessageCollection.prototype, "infos", {
        /**
         * Gets the list of all the info validation messages.
         *
         * @readonly
         */
        get: function () {
            return new ValidationMessageCollection(this.validationMessages.filter(function (validationMessage) { return validationMessage.validationLevel === validation_level_1.ValidationLevel.info; }));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidationMessageCollection.prototype, "length", {
        /**
         * Gets the length of all the validation messages.
         *
         * @readonly
         */
        get: function () {
            return this.validationMessages.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidationMessageCollection.prototype, "warnings", {
        /**
         * Gets the list of all the warning validation messages.
         *
         * @readonly
         */
        get: function () {
            return new ValidationMessageCollection(this.validationMessages.filter(function (validationMessage) { return validationMessage.validationLevel === validation_level_1.ValidationLevel.warning; }));
        },
        enumerable: true,
        configurable: true
    });
    // #endregion
    // #region Public Methods (4)
    /**
     * Filters validation messages by property name.
     *
     * @param {string} propertyName
     * @returns - The validation message collection.
     */
    ValidationMessageCollection.prototype.filter = function (propertyName) {
        defensive_programming_framework_1.cannotBeNullOrEmpty(propertyName);
        return new ValidationMessageCollection(this.validationMessages.filter(function (validationMessage, i, array) { return validationMessage.propertyName === propertyName; }));
    };
    /**
     * Gets the validation message by the specified index.
     *
     * @param {number} index - The index.
     * @returns - The validation message at the specified index or null if index is outside of bounds of the array.
     */
    ValidationMessageCollection.prototype.get = function (index) {
        defensive_programming_framework_1.mustBeInteger(index);
        if (index >= 0 &&
            index < this.validationMessages.length) {
            return this.validationMessages[index];
        }
        else {
            return null;
        }
    };
    /**
     * Converts validation messages as an array.
     *
     * @returns - The array representation of the validation messages collection.
     */
    ValidationMessageCollection.prototype.toArray = function () {
        return this.validationMessages.concat([]);
    };
    /**
     * Converts validation messages to a string.
     *
     * @returns - The string representation of the validation message collection.
     */
    ValidationMessageCollection.prototype.toString = function () {
        return this.validationMessages.map(function (x) { return x.message; }).join("\n");
    };
    // #endregion
    // #region Private Methods ()
    /**
     * Compares two numbers.
     *
     * @private
     * @param {number} a - The first number.
     * @param {number} b - The second number.
     * @returns - 1 if a > b, -1 if b < a, 0 otherwise.
     */
    ValidationMessageCollection.prototype.compare = function (a, b) {
        if (a > b) {
            return 1;
        }
        else if (a < b) {
            return -1;
        }
        else {
            return 0;
        }
    };
    /**
     * Converts validation level enum value to number.
     *
     * @private
     * @param {ValidationLevel} validationLevel - The validation level.
     * @returns - The validation level number value.
     */
    ValidationMessageCollection.prototype.validationLevelToNumber = function (validationLevel) {
        if (validationLevel === validation_level_1.ValidationLevel.error) {
            return 3;
        }
        else if (validationLevel === validation_level_1.ValidationLevel.warning) {
            return 2;
        }
        else {
            return 1;
        }
    };
    return ValidationMessageCollection;
}());
exports.ValidationMessageCollection = ValidationMessageCollection;
//# sourceMappingURL=validation-message-collection.js.map