import { ValidationLevel } from "../validation-level";
/**
 * The validator base class.
 *
 * @export
 * @abstract
 * @class Validator
 */
export declare abstract class Validator {
    /**
     * The message template.
     *
     * @private
     * @type {(string | null | undefined)}
     */
    private readonly messageTemplate;
    /**
     * The  validation context.
     *
     * @type {(string | null | undefined)}
     */
    readonly validationContext: string | null | undefined;
    /**
     * The validation level.
     *
     * @type {ValidationLevel}
     */
    readonly validationLevel: ValidationLevel;
    /**
     * The validation property.
     *
     * @type {number}
     */
    readonly validationPriority: number;
    /**
     * The get localized validation message template delegate.
     *
     * @static
     */
    static getLocalizedValidationMessage: (validationMessageTemplate: string) => string | null;
    /**
     * Creates an instance of Validator.
     * @param {(string | null | undefined)} message - The custom validation message.
     * @param {(ValidationLevel | null | undefined)} validationLevel - The custom validation level.
     * @param {(string | null | undefined)} validationContext - The custom validation context.
     * @param {(number | null | undefined)} validationPriority - The custom validation priority.
     */
    constructor(message: string | null | undefined, validationLevel: ValidationLevel | null | undefined, validationContext: string | null | undefined, validationPriority: number | null | undefined);
    /**
     * Gets the validation message.
     *
     * @readonly
     * @type {string}
     */
    readonly message: string;
    /**
    * Validates the specified value.
    *
    * @param {*} value
    * @returns {boolean} - True if the value is valid; false otherwise.
    */
    abstract isValid(value: any): boolean;
    /**
     * Gets the message parameters.
     *
     * @protected
     * @returns {string} - The message parameters
     */
    protected getMessageParameters(): any[];
    /**
     * Gets the default message.
     *
     * @protected
     * @abstract
     * @returns {string} - The default message.
     */
    protected abstract getDefaultMessage(): string;
    /**
     * Replaces the placeholders in the message template with actual values.
     *
     * @private
     * @param {string} messageTemplate - The message template.
     * @param {any[]} messageParameters - The message parameters.
     * @returns
     */
    private formatMessage;
    /**
     * Localizes the validation message if the get localized message delegate that provides the localized message template is set.
     *
     * @private
     * @param {string} messageTemplate - The message template.
     * @returns - The localized message.
     */
    private localizeMessage;
    /**
     * Formats the string by replacing the placeholders within the string with actual values.
     *
     * @private
     * @param {string} str - The template string.
     * @param {string[]} args - The argument values.
     * @returns
     */
    private stringFormat;
    /**
     * Converts the specified value to string.
     *
     * @private
     * @param {*} value - The value.
     * @returns {string} - The string representation of the value.
     */
    private toString;
}
