"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var validation_context_1 = require("../validation-context");
var validation_level_1 = require("../validation-level");
var defensive_programming_framework_1 = require("defensive-programming-framework");
/**
 * The validator base class.
 *
 * @export
 * @abstract
 * @class Validator
 */
var Validator = /** @class */ (function () {
    // #endregion
    // #region Constructors (1)
    /**
     * Creates an instance of Validator.
     * @param {(string | null | undefined)} message - The custom validation message.
     * @param {(ValidationLevel | null | undefined)} validationLevel - The custom validation level.
     * @param {(string | null | undefined)} validationContext - The custom validation context.
     * @param {(number | null | undefined)} validationPriority - The custom validation priority.
     */
    function Validator(message, validationLevel, validationContext, validationPriority) {
        this.messageTemplate = message;
        this.validationContext = defensive_programming_framework_1.whenIsNull(validationContext, validation_context_1.ValidationContext.default);
        this.validationLevel = defensive_programming_framework_1.whenIsNull(validationLevel, validation_level_1.ValidationLevel.error);
        this.validationPriority = defensive_programming_framework_1.whenIsNull(validationPriority, 0);
    }
    Object.defineProperty(Validator.prototype, "message", {
        // #endregion
        // #region Public Accessors (1)
        /**
         * Gets the validation message.
         *
         * @readonly
         * @type {string}
         */
        get: function () {
            var messageTemplate = defensive_programming_framework_1.whenIsNullOrEmpty(defensive_programming_framework_1.whenIsNullOrEmpty(this.messageTemplate, this.getDefaultMessage()), "Undefined message.");
            var localizedMessage = this.localizeMessage(messageTemplate) || messageTemplate;
            var formattedMessage = this.formatMessage(localizedMessage, this.getMessageParameters()) || messageTemplate;
            return formattedMessage;
        },
        enumerable: true,
        configurable: true
    });
    // #endregion
    // #region Protected Methods (1)
    /**
     * Gets the message parameters.
     *
     * @protected
     * @returns {string} - The message parameters
     */
    Validator.prototype.getMessageParameters = function () {
        return [];
    };
    // #endregion
    // #region Private Methods (4)
    /**
     * Replaces the placeholders in the message template with actual values.
     *
     * @private
     * @param {string} messageTemplate - The message template.
     * @param {any[]} messageParameters - The message parameters.
     * @returns
     */
    Validator.prototype.formatMessage = function (messageTemplate, messageParameters) {
        var _this = this;
        defensive_programming_framework_1.cannotBeNullOrEmpty(messageTemplate);
        if (!defensive_programming_framework_1.isNullOrEmptyArray(messageParameters)) {
            try {
                messageTemplate = this.stringFormat(messageTemplate, messageParameters.map(function (x) { return _this.toString(x); }));
            }
            catch (ex) {
            }
        }
        return messageTemplate;
    };
    /**
     * Localizes the validation message if the get localized message delegate that provides the localized message template is set.
     *
     * @private
     * @param {string} messageTemplate - The message template.
     * @returns - The localized message.
     */
    Validator.prototype.localizeMessage = function (messageTemplate) {
        return defensive_programming_framework_1.isNull(Validator.getLocalizedValidationMessage) ? null : Validator.getLocalizedValidationMessage(messageTemplate);
    };
    /**
     * Formats the string by replacing the placeholders within the string with actual values.
     *
     * @private
     * @param {string} str - The template string.
     * @param {string[]} args - The argument values.
     * @returns
     */
    Validator.prototype.stringFormat = function (str, args) {
        return str.replace(/{(\d+)}/g, function (match, index) { return args[index] || ""; });
    };
    /**
     * Converts the specified value to string.
     *
     * @private
     * @param {*} value - The value.
     * @returns {string} - The string representation of the value.
     */
    Validator.prototype.toString = function (value) {
        var _this = this;
        if (value === null) {
            return "null";
        }
        else if (value === undefined) {
            return "undefined";
        }
        else if (typeof value === "function") {
            return value.name;
        }
        else if (value instanceof Array) {
            if (value.length <= 10) {
                return "[" + value.map(function (x) { return _this.toString(x); }).join(", ") + "]";
            }
            else {
                return "[" + value.filter(function (u, i) { return i < 10; }).map(function (x) { return _this.toString(x); }).join(", ") + ", ...]";
            }
        }
        else {
            return value.toString();
        }
    };
    /**
     * The get localized validation message template delegate.
     *
     * @static
     */
    Validator.getLocalizedValidationMessage = function (message) { return null; };
    return Validator;
}());
exports.Validator = Validator;
//# sourceMappingURL=validator.js.map